'use strict';

var ReactDOM = require('react-dom');
var react = require('react');
var shim = require('use-sync-external-store/shim');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var ReactDOM__default = /*#__PURE__*/_interopDefaultLegacy(ReactDOM);

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var run = function (fn) { return fn(); };
var batch = ReactDOM__default["default"].unstable_batchedUpdates || /* c8 ignore next */ run;
var __DEV__ = process.env.NODE_ENV !== 'production';
function create(initStore) {
    if (__DEV__ && typeof initStore !== 'function') {
        throw new Error('Expected a function');
    }
    var store;
    var listeners = new Set();
    function getSetStore(s) {
        if (typeof s === 'undefined')
            return store;
        var partial = typeof s === 'function' ? s(store) : s;
        store = __assign(__assign({}, store), partial);
        batch(function () { return listeners.forEach(function (listener) { return listener(partial); }); });
    }
    /* c8 ignore start */
    getSetStore.get = function () {
        console.error('get() is deprecated since v6, upgrade: https://github.com/nanxiaobei/flooks');
        return {};
    };
    getSetStore.set = function () {
        console.error('set() is deprecated since v6, upgrade: https://github.com/nanxiaobei/flooks');
    };
    /* c8 ignore stop */
    store = initStore(getSetStore);
    return function useStore() {
        var proxy = react.useRef({});
        var handler = react.useRef({});
        var hasState = react.useRef(false);
        var hasUpdate = react.useRef(false);
        var _a = react.useState(false), setState = _a[1];
        react.useMemo(function () {
            handler.current = {
                get: function (target, key) {
                    var val = store[key];
                    if (typeof val !== 'function') {
                        hasState.current = true;
                        target[key] = val;
                        return target[key];
                    }
                    target[key] = new Proxy(val, {
                        get: function (fn, fnKey) {
                            if (fnKey === 'loading' && !('loading' in fn)) {
                                fn.loading = false;
                            }
                            return fn[fnKey];
                        },
                        apply: function (fn, _this, args) {
                            var res = fn.apply(void 0, args);
                            if (!('loading' in fn) ||
                                !res ||
                                typeof res.then !== 'function') {
                                target[key] = fn;
                                return res;
                            }
                            var setLoading = function (loading) {
                                target[key].loading = loading;
                                setState(function (s) { return !s; });
                            };
                            target[key] = (function () {
                                var newArgs = [];
                                for (var _i = 0; _i < arguments.length; _i++) {
                                    newArgs[_i] = arguments[_i];
                                }
                                var newRes = fn.apply(void 0, newArgs);
                                setLoading(true);
                                return newRes.finally(function () { return setLoading(false); });
                            });
                            setLoading(true);
                            return res.finally(function () { return setLoading(false); });
                        },
                    });
                    return target[key];
                },
                set: function (target, key, val) {
                    if (key in target && val !== target[key]) {
                        hasUpdate.current = true;
                        target[key] = val;
                    }
                    return true;
                },
            };
            proxy.current = new Proxy({}, handler.current);
        }, []);
        react.useEffect(function () {
            handler.current.get = function (target, key) { return target[key]; };
        }, []);
        var subscribe = react.useCallback(function (update) {
            if (!hasState.current)
                return function () { return undefined; };
            var listener = function (partial) {
                Object.assign(proxy.current, partial);
                if (hasUpdate.current) {
                    hasUpdate.current = false;
                    update();
                }
            };
            listeners.add(listener);
            return function () { return listeners.delete(listener); };
        }, []);
        var getSnapshot = react.useCallback(function () { return store; }, []);
        shim.useSyncExternalStore(subscribe, getSnapshot);
        return proxy.current;
    };
}

module.exports = create;
